@@ -6,15 +6,6 @@
 use super::*;
 
 impl StateHub {
-    // async fn get_states_helper(&self, names: &[v1::StateName]) -> anyhow::Result<Vec<v1::State>> {
-    //     let mut states = vec![];
-    //     for name in names {
-    //         let state = self.api.get_state(name).await?.into_inner()?;
-    //         states.push(state);
-    //     }
-    //     Ok(states)
-    // }
-
     pub(super) async fn add_location_helper(
         &self,
         state: &v1::State,
@@ -82,15 +73,15 @@ impl StateHub {
 
         log::info!("Checking {}", state.show());
         for location in locations {
-            if !state.is_available_in(location) {
-                self.inform(format!("Extdending state {} to {}", state.name, location));
-                self.add_location_helper(&state, location).await?;
-            } else {
+            if state.is_available_in(location) {
                 log::info!(
                     "Skipping state {} which is already available in {}",
                     state.name,
                     location
                 );
+            } else {
+                self.inform(format!("Extdending state {} to {}", state.name, location))?;
+                self.add_location_helper(&state, location).await?;
             }
         }
 
@@ -115,12 +106,28 @@ impl StateHub {
         helm: &k8s::Helm,
     ) -> anyhow::Result<()> {
         let token = self.api.issue_cluster_token(&cluster.name).await?;
-        self.verbosely(format!("Issued token {} for {}", token.token, cluster));
+        self.verbosely(format!("Issued token {} for {}", token.token, cluster))?;
         let namespace = helm.namespace();
         k8s::store_cluster_token(namespace, &token.token).await?;
         Ok(())
     }
 
+    pub(super) async fn claim_unowned_states_helper(
+        &self,
+        cluster: &v1::Cluster,
+        states: Option<Vec<v1::StateName>>,
+    ) -> anyhow::Result<()> {
+        if let Some(states) = states {
+            for state in states {
+                if self.api.get_state(&state).await?.owner.is_none() {
+                    self.verbosely(format!("Claiming ownership of state {}", state))?;
+                    self.api.set_owner(state, &cluster.name).await?;
+                }
+            }
+        }
+        Ok(())
+    }
+
     pub(super) async fn relinquish_states_helper(
         &self,
         cluster: &v1::ClusterName,
